@@ -20,7 +20,7 @@
 
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
-#include <SkXfermode.h>
+#include <SkBlendMode.h>
 #include <memory>
 
 namespace android {
@@ -36,10 +36,18 @@ public:
     };
     void syncEnabled();
 
-    static void getFactors(SkXfermode::Mode mode, ModeOrderSwap modeUsage,
+    static void getFactors(SkBlendMode mode, ModeOrderSwap modeUsage,
             GLenum* outSrc, GLenum* outDst);
     void setFactors(GLenum src, GLenum dst);
 
+    bool  getEnabled() {
+        return mEnabled;
+    }
+    void getFactors(GLenum* src, GLenum* dst) {
+        *src = mSrcMode;
+        *dst = mDstMode;
+    }
+
     void dump();
 private:
     Blend();
