@@ -31,13 +31,12 @@ import android.view.ActionMode;
 import android.view.Gravity;
 import android.view.KeyEvent;
 import android.view.View;
-import android.view.View.OnFocusChangeListener;
 import android.view.ViewDebug;
 import android.view.ViewGroup;
 import android.view.ViewHierarchyEncoder;
-import android.view.ViewParent;
 import android.view.Window;
 import android.widget.SpinnerAdapter;
+
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 
@@ -110,6 +109,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public static final int NAVIGATION_MODE_STANDARD = 0;
     
     /**
@@ -122,6 +122,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public static final int NAVIGATION_MODE_LIST = 1;
     
     /**
@@ -133,6 +134,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public static final int NAVIGATION_MODE_TABS = 2;
 
     /** @hide */
@@ -340,6 +342,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void setListNavigationCallbacks(SpinnerAdapter adapter,
             OnNavigationListener callback);
 
@@ -353,6 +356,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void setSelectedNavigationItem(int position);
 
     /**
@@ -365,6 +369,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract int getSelectedNavigationIndex();
 
     /**
@@ -377,6 +382,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract int getNavigationItemCount();
 
     /**
@@ -579,6 +585,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     @NavigationMode
     public abstract int getNavigationMode();
 
@@ -595,6 +602,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void setNavigationMode(@NavigationMode int mode);
 
     /**
@@ -621,6 +629,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract Tab newTab();
 
     /**
@@ -634,6 +643,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void addTab(Tab tab);
 
     /**
@@ -647,6 +657,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void addTab(Tab tab, boolean setSelected);
 
     /**
@@ -662,6 +673,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void addTab(Tab tab, int position);
 
     /**
@@ -677,6 +689,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void addTab(Tab tab, int position, boolean setSelected);
 
     /**
@@ -690,6 +703,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void removeTab(Tab tab);
 
     /**
@@ -703,6 +717,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void removeTabAt(int position);
 
     /**
@@ -713,6 +728,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void removeAllTabs();
 
     /**
@@ -727,6 +743,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract void selectTab(Tab tab);
 
     /**
@@ -740,6 +757,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract Tab getSelectedTab();
 
     /**
@@ -753,6 +771,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract Tab getTabAt(int index);
 
     /**
@@ -764,6 +783,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public abstract int getTabCount();
 
     /**
@@ -1048,6 +1068,11 @@ public abstract class ActionBar {
         return false;
     }
 
+    /** @hide */
+    public boolean closeOptionsMenu() {
+        return false;
+    }
+
     /** @hide */
     public boolean invalidateOptionsMenu() {
         return false;
@@ -1072,66 +1097,10 @@ public abstract class ActionBar {
     public void setWindowTitle(CharSequence title) {
     }
 
-    /**
-     * Attempts to move focus to the ActionBar if it does not already contain the focus.
-     *
-     * @return {@code true} if focus changes or {@code false} if focus doesn't change.
-     * @hide
-     */
-    public boolean requestFocus() {
-        return false;
-    }
-
     /** @hide */
     public void onDestroy() {
     }
 
-    /**
-     * Common implementation for requestFocus that takes in the Toolbar and moves focus
-     * to the contents. This makes the ViewGroups containing the toolbar allow focus while it stays
-     * in the ActionBar and then prevents it again once it leaves.
-     *
-     * @param viewGroup The toolbar ViewGroup
-     * @return {@code true} if focus changes or {@code false} if focus doesn't change.
-     * @hide
-     */
-    protected boolean requestFocus(ViewGroup viewGroup) {
-        if (viewGroup != null && !viewGroup.hasFocus()) {
-            final ViewGroup toolbar = viewGroup.getTouchscreenBlocksFocus() ? viewGroup : null;
-            ViewParent parent = viewGroup.getParent();
-            ViewGroup container = null;
-            while (parent != null && parent instanceof ViewGroup) {
-                final ViewGroup vgParent = (ViewGroup) parent;
-                if (vgParent.getTouchscreenBlocksFocus()) {
-                    container = vgParent;
-                    break;
-                }
-                parent = vgParent.getParent();
-            }
-            if (container != null) {
-                container.setTouchscreenBlocksFocus(false);
-            }
-            if (toolbar != null) {
-                toolbar.setTouchscreenBlocksFocus(false);
-            }
-            viewGroup.requestFocus();
-            final View focused = viewGroup.findFocus();
-            if (focused != null) {
-                focused.setOnFocusChangeListener(new FollowOutOfActionBar(viewGroup,
-                        container, toolbar));
-            } else {
-                if (container != null) {
-                    container.setTouchscreenBlocksFocus(true);
-                }
-                if (toolbar != null) {
-                    toolbar.setTouchscreenBlocksFocus(true);
-                }
-            }
-            return true;
-        }
-        return false;
-    }
-
     /**
      * Listener interface for ActionBar navigation events.
      *
@@ -1140,6 +1109,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public interface OnNavigationListener {
         /**
          * This method is called whenever a navigation item in your action bar
@@ -1177,6 +1147,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public static abstract class Tab {
         /**
          * An invalid position for a tab.
@@ -1333,6 +1304,7 @@ public abstract class ActionBar {
      * <a href="http://developer.android.com/design/patterns/navigation.html">common
      * navigation patterns</a> instead.
      */
+    @Deprecated
     public interface TabListener {
         /**
          * Called when a tab enters the selected state.
@@ -1450,43 +1422,4 @@ public abstract class ActionBar {
             encoder.addProperty("gravity", gravity);
         }
     }
-
-    /**
-     * Tracks the focused View until it leaves the ActionBar, then it resets the
-     * touchscreenBlocksFocus value.
-     */
-    private static class FollowOutOfActionBar implements OnFocusChangeListener, Runnable {
-        private final ViewGroup mFocusRoot;
-        private final ViewGroup mContainer;
-        private final ViewGroup mToolbar;
-
-        public FollowOutOfActionBar(ViewGroup focusRoot, ViewGroup container, ViewGroup toolbar) {
-            mContainer = container;
-            mToolbar = toolbar;
-            mFocusRoot = focusRoot;
-        }
-
-        @Override
-        public void onFocusChange(View v, boolean hasFocus) {
-            if (!hasFocus) {
-                v.setOnFocusChangeListener(null);
-                final View focused = mFocusRoot.findFocus();
-                if (focused != null) {
-                    focused.setOnFocusChangeListener(this);
-                } else {
-                    mFocusRoot.post(this);
-                }
-            }
-        }
-
-        @Override
-        public void run() {
-            if (mContainer != null) {
-                mContainer.setTouchscreenBlocksFocus(true);
-            }
-            if (mToolbar != null) {
-                mToolbar.setTouchscreenBlocksFocus(true);
-            }
-        }
-    }
 }
