@@ -321,10 +321,10 @@ public class BluetoothGattCharacteristic implements Parcelable {
     }
 
     /**
-     * Returns the deisred key size.
+     * Returns the desired key size.
      * @hide
      */
-    /*package*/ int getKeySize() {
+    public int getKeySize() {
         return mKeySize;
     }
 
@@ -392,6 +392,14 @@ public class BluetoothGattCharacteristic implements Parcelable {
         return mInstance;
     }
 
+    /**
+     * Force the instance ID.
+     * @hide
+     */
+    public void setInstanceId(int instanceId) {
+        mInstance = instanceId;
+    }
+
     /**
      * Returns the properties of this characteristic.
      *
