@@ -226,6 +226,14 @@ public class BluetoothGattDescriptor implements Parcelable {
         return mInstance;
     }
 
+    /**
+     * Force the instance ID.
+     * @hide
+     */
+    public void setInstanceId(int instanceId) {
+        mInstance = instanceId;
+    }
+
     /**
      * Returns the permissions for this descriptor.
      *
