@@ -23,6 +23,8 @@ import android.os.Parcel;
 import android.os.ParcelUuid;
 import android.os.Parcelable;
 
+import com.android.internal.util.BitUtils;
+
 import java.util.Arrays;
 import java.util.List;
 import java.util.Objects;
@@ -67,7 +69,9 @@ public final class ScanFilter implements Parcelable {
     private final byte[] mManufacturerData;
     @Nullable
     private final byte[] mManufacturerDataMask;
-    private static final ScanFilter EMPTY = new ScanFilter.Builder().build() ;
+
+    /** @hide */
+    public static final ScanFilter EMPTY = new ScanFilter.Builder().build() ;
 
 
     private ScanFilter(String name, String deviceAddress, ParcelUuid uuid,
@@ -318,8 +322,12 @@ public final class ScanFilter implements Parcelable {
         return true;
     }
 
-    // Check if the uuid pattern is contained in a list of parcel uuids.
-    private boolean matchesServiceUuids(ParcelUuid uuid, ParcelUuid parcelUuidMask,
+    /**
+     * Check if the uuid pattern is contained in a list of parcel uuids.
+     *
+     * @hide
+     */
+    public static boolean matchesServiceUuids(ParcelUuid uuid, ParcelUuid parcelUuidMask,
             List<ParcelUuid> uuids) {
         if (uuid == null) {
             return true;
@@ -338,16 +346,8 @@ public final class ScanFilter implements Parcelable {
     }
 
     // Check if the uuid pattern matches the particular service uuid.
-    private boolean matchesServiceUuid(UUID uuid, UUID mask, UUID data) {
-        if (mask == null) {
-            return uuid.equals(data);
-        }
-        if ((uuid.getLeastSignificantBits() & mask.getLeastSignificantBits()) !=
-                (data.getLeastSignificantBits() & mask.getLeastSignificantBits())) {
-            return false;
-        }
-        return ((uuid.getMostSignificantBits() & mask.getMostSignificantBits()) ==
-                (data.getMostSignificantBits() & mask.getMostSignificantBits()));
+    private static boolean matchesServiceUuid(UUID uuid, UUID mask, UUID data) {
+        return BitUtils.maskedEquals(data, uuid, mask);
     }
 
     // Check whether the data pattern matches the parsed data.
