@@ -16,21 +16,19 @@
 
 package android.graphics;
 
+/**
+ * <p>Specialized implementation of {@link Paint}'s
+ * {@link Paint#setXfermode(Xfermode) transfer mode}. Refer to the
+ * documentation of the {@link PorterDuff.Mode} enum for more
+ * information on the available alpha compositing and blending modes.</p>
+ */
 public class PorterDuffXfermode extends Xfermode {
-    /**
-     * @hide
-     */
-    public final PorterDuff.Mode mode;
-
     /**
      * Create an xfermode that uses the specified porter-duff mode.
      *
      * @param mode           The porter-duff mode that is applied
      */
     public PorterDuffXfermode(PorterDuff.Mode mode) {
-        this.mode = mode;
-        native_instance = nativeCreateXfermode(mode.nativeInt);
+        porterDuffMode = mode.nativeInt;
     }
-    
-    private static native long nativeCreateXfermode(int mode);
 }
