@@ -47,6 +47,7 @@ public final class GnssMeasurement implements Parcelable {
     private double mCarrierPhaseUncertainty;
     private int mMultipathIndicator;
     private double mSnrInDb;
+    private double mAutomaticGainControlLevelInDb;
 
     // The following enumerations must be in sync with the values declared in gps.h
 
@@ -56,6 +57,7 @@ public final class GnssMeasurement implements Parcelable {
     private static final int HAS_CARRIER_CYCLES = (1<<10);
     private static final int HAS_CARRIER_PHASE = (1<<11);
     private static final int HAS_CARRIER_PHASE_UNCERTAINTY = (1<<12);
+    private static final int HAS_AUTOMATIC_GAIN_CONTROL = (1<<13);
 
     /**
      * The status of the multipath indicator.
@@ -63,7 +65,7 @@ public final class GnssMeasurement implements Parcelable {
      */
     @Retention(RetentionPolicy.SOURCE)
     @IntDef({MULTIPATH_INDICATOR_UNKNOWN, MULTIPATH_INDICATOR_DETECTED,
-        MULTIPATH_INDICATOR_NOT_USED})
+            MULTIPATH_INDICATOR_NOT_DETECTED})
     public @interface MultipathIndicator {}
 
     /**
@@ -81,9 +83,6 @@ public final class GnssMeasurement implements Parcelable {
      */
     public static final int MULTIPATH_INDICATOR_NOT_DETECTED = 2;
 
-    /** @removed */
-    public static final int MULTIPATH_INDICATOR_NOT_USED = 2;
-
     /** This GNSS measurement's tracking state is invalid or unknown. */
     public static final int STATE_UNKNOWN = 0;
     /** This GNSS measurement's tracking state has code lock. */
@@ -114,6 +113,18 @@ public final class GnssMeasurement implements Parcelable {
     public static final int STATE_GAL_E1B_PAGE_SYNC = (1<<12);
     /** This SBAS measurement's tracking state has whole second level sync. */
     public static final int STATE_SBAS_SYNC = (1<<13);
+    /**
+     * This GNSS measurement's tracking state has time-of-week known, possibly not decoded
+     * over the air but has been determined from other sources. If TOW decoded is set then TOW Known
+     * will also be set.
+     */
+    public static final int STATE_TOW_KNOWN = (1<<14);
+    /**
+     * This Glonass measurement's tracking state has time-of-day known, possibly not decoded
+     * over the air but has been determined from other sources. If TOD decoded is set then TOD Known
+     * will also be set.
+     */
+    public static final int STATE_GLO_TOD_KNOWN = (1<<15);
 
     /**
      * All the GNSS receiver state flags, for bit masking purposes (not a sensible state for any
@@ -183,6 +194,7 @@ public final class GnssMeasurement implements Parcelable {
         mCarrierPhaseUncertainty = measurement.mCarrierPhaseUncertainty;
         mMultipathIndicator = measurement.mMultipathIndicator;
         mSnrInDb = measurement.mSnrInDb;
+        mAutomaticGainControlLevelInDb = measurement.mAutomaticGainControlLevelInDb;
     }
 
     /**
@@ -302,6 +314,9 @@ public final class GnssMeasurement implements Parcelable {
         if ((mState & STATE_TOW_DECODED) != 0) {
             builder.append("TowDecoded|");
         }
+        if ((mState & STATE_TOW_KNOWN) != 0) {
+          builder.append("TowKnown|");
+        }
         if ((mState & STATE_MSEC_AMBIGUOUS) != 0) {
             builder.append("MsecAmbiguous|");
         }
@@ -314,6 +329,9 @@ public final class GnssMeasurement implements Parcelable {
         if ((mState & STATE_GLO_TOD_DECODED) != 0) {
             builder.append("GloTodDecoded|");
         }
+        if ((mState & STATE_GLO_TOD_KNOWN) != 0) {
+          builder.append("GloTodKnown|");
+        }
         if ((mState & STATE_BDS_D2_BIT_SYNC) != 0) {
             builder.append("BdsD2BitSync|");
         }
@@ -359,10 +377,14 @@ public final class GnssMeasurement implements Parcelable {
      *     C/A code lock   : [ 0   1ms ]   : STATE_CODE_LOCK is set
      *     Bit sync        : [ 0  20ms ]   : STATE_BIT_SYNC is set
      *     Subframe sync   : [ 0    6s ]   : STATE_SUBFRAME_SYNC is set
-     *     TOW decoded     : [ 0 1week ]   : STATE_TOW_DECODED is set</pre>
+     *     TOW decoded     : [ 0 1week ]   : STATE_TOW_DECODED is set
+     *     TOW Known       : [ 0 1week ]   : STATE_TOW_KNOWN set</pre>
+     *
+     * Note: TOW Known refers to the case where TOW is possibly not decoded over the air but has
+     * been determined from other sources. If TOW decoded is set then TOW Known must also be set.
      *
      * <p>Note well: if there is any ambiguity in integer millisecond, {@code STATE_MSEC_AMBIGUOUS}
-     * should be set accordingly, in the 'state' field.
+     * must be set accordingly, in the 'state' field.
      *
      * <p>This value must be populated if 'state' != {@code STATE_UNKNOWN}.
      *
@@ -374,12 +396,17 @@ public final class GnssMeasurement implements Parcelable {
      * <p>Given the highest sync state that can be achieved, per each satellite, valid range for
      * this field can be:
      * <pre>
-     *     Searching       : [ 0       ]   : STATE_UNKNOWN
-     *     C/A code lock   : [ 0   1ms ]   : STATE_CODE_LOCK is set
-     *     Symbol sync     : [ 0  10ms ]   : STATE_SYMBOL_SYNC is set
-     *     Bit sync        : [ 0  20ms ]   : STATE_BIT_SYNC is set
-     *     String sync     : [ 0    2s ]   : STATE_GLO_STRING_SYNC is set
-     *     Time of day     : [ 0  1day ]   : STATE_GLO_TOD_DECODED is set</pre>
+     *     Searching           : [ 0       ]   : STATE_UNKNOWN
+     *     C/A code lock       : [ 0   1ms ]   : STATE_CODE_LOCK is set
+     *     Symbol sync         : [ 0  10ms ]   : STATE_SYMBOL_SYNC is set
+     *     Bit sync            : [ 0  20ms ]   : STATE_BIT_SYNC is set
+     *     String sync         : [ 0    2s ]   : STATE_GLO_STRING_SYNC is set
+     *     Time of day decoded : [ 0  1day ]   : STATE_GLO_TOD_DECODED is set
+     *     Time of day known   : [ 0  1day ]   : STATE_GLO_TOD_KNOWN set</pre>
+     *
+     * Note: Time of day known refers to the case where it is possibly not decoded over the air but
+     * has been determined from other sources. If Time of day decoded is set then Time of day known
+     * must also be set.
      *
      * <p>For Beidou, this is:
      * <ul>
@@ -389,23 +416,31 @@ public final class GnssMeasurement implements Parcelable {
      * <p>Given the highest sync state that can be achieved, per each satellite, valid range for
      * this field can be:
      * <pre>
-     *     Searching       : [ 0       ]   : STATE_UNKNOWN
-     *     C/A code lock   : [ 0   1ms ]   : STATE_CODE_LOCK is set
-     *     Bit sync (D2)   : [ 0   2ms ]   : STATE_BDS_D2_BIT_SYNC is set
-     *     Bit sync (D1)   : [ 0  20ms ]   : STATE_BIT_SYNC is set
-     *     Subframe (D2)   : [ 0  0.6s ]   : STATE_BDS_D2_SUBFRAME_SYNC is set
-     *     Subframe (D1)   : [ 0    6s ]   : STATE_SUBFRAME_SYNC is set
-     *     Time of week    : [ 0 1week ]   : STATE_TOW_DECODED is set</pre>
+     *     Searching              : [ 0       ]   : STATE_UNKNOWN
+     *     C/A code lock          : [ 0   1ms ]   : STATE_CODE_LOCK is set
+     *     Bit sync (D2)          : [ 0   2ms ]   : STATE_BDS_D2_BIT_SYNC is set
+     *     Bit sync (D1)          : [ 0  20ms ]   : STATE_BIT_SYNC is set
+     *     Subframe (D2)          : [ 0  0.6s ]   : STATE_BDS_D2_SUBFRAME_SYNC is set
+     *     Subframe (D1)          : [ 0    6s ]   : STATE_SUBFRAME_SYNC is set
+     *     Time of week decoded   : [ 0 1week ]   : STATE_TOW_DECODED is set
+     *     Time of week known     : [ 0 1week ]   : STATE_TOW_KNOWN set</pre>
+     *
+     * Note: TOW Known refers to the case where TOW is possibly not decoded over the air but has
+     * been determined from other sources. If TOW decoded is set then TOW Known must also be set.
      *
      * <p>For Galileo, this is:
      * <ul>
      * <li>Received Galileo time of week, at the measurement time in nanoseconds.</li>
      * </ul>
      * <pre>
-     *     E1BC code lock   : [ 0   4ms ]  : STATE_GAL_E1BC_CODE_LOCK is set
-     *     E1C 2nd code lock: [ 0 100ms ]  : STATE_GAL_E1C_2ND_CODE_LOCK is set
-     *     E1B page         : [ 0    2s ]  : STATE_GAL_E1B_PAGE_SYNC is set
-     *     Time of week     : [ 0 1week ]  : STATE_GAL_TOW_DECODED is set</pre>
+     *     E1BC code lock       : [ 0   4ms ]  : STATE_GAL_E1BC_CODE_LOCK is set
+     *     E1C 2nd code lock    : [ 0 100ms ]  : STATE_GAL_E1C_2ND_CODE_LOCK is set
+     *     E1B page             : [ 0    2s ]  : STATE_GAL_E1B_PAGE_SYNC is set
+     *     Time of week decoded : [ 0 1week ]  : STATE_TOW_DECODED is set
+     *     Time of week known   : [ 0 1week ]  : STATE_TOW_KNOWN set</pre>
+     *
+     * Note: TOW Known refers to the case where TOW is possibly not decoded over the air but has
+     * been determined from other sources. If TOW decoded is set then TOW Known must also be set.
      *
      * <p>For SBAS, this is:
      * <ul>
@@ -623,19 +658,27 @@ public final class GnssMeasurement implements Parcelable {
     }
 
     /**
-     * Gets the carrier frequency at which codes and messages are modulated.
+     * Gets the carrier frequency of the tracked signal.
      *
-     * <p>For GPS, e.g., it can be L1 or L2.  If the field is not set, it is the primary common use
-     * frequency, e.g. L1 for GPS.
+     * <p>For example it can be the GPS central frequency for L1 = 1575.45 MHz, or L2 = 1227.60 MHz,
+     * L5 = 1176.45 MHz, varying GLO channels, etc. If the field is not set, it is the primary
+     * common use central frequency, e.g. L1 = 1575.45 MHz for GPS.
+     *
+     * For an L1, L5 receiver tracking a satellite on L1 and L5 at the same time, two raw
+     * measurement objects will be reported for this same satellite, in one of the measurement
+     * objects, all the values related to L1 will be filled, and in the other all of the values
+     * related to L5 will be filled.
      *
      * <p>The value is only available if {@link #hasCarrierFrequencyHz()} is {@code true}.
+     *
+     * @return the carrier frequency of the signal tracked in Hz.
      */
     public float getCarrierFrequencyHz() {
         return mCarrierFrequencyHz;
     }
 
     /**
-     * Sets the Carrier frequency (L1 or L2) in Hz.
+     * Sets the Carrier frequency in Hz.
      * @hide
      */
     @TestApi
@@ -645,7 +688,7 @@ public final class GnssMeasurement implements Parcelable {
     }
 
     /**
-     * Resets the Carrier frequency (L1 or L2) in Hz.
+     * Resets the Carrier frequency in Hz.
      * @hide
      */
     @TestApi
@@ -803,8 +846,8 @@ public final class GnssMeasurement implements Parcelable {
                 return "Unknown";
             case MULTIPATH_INDICATOR_DETECTED:
                 return "Detected";
-            case MULTIPATH_INDICATOR_NOT_USED:
-                return "NotUsed";
+            case MULTIPATH_INDICATOR_NOT_DETECTED:
+                return "NotDetected";
             default:
                 return "<Invalid:" + mMultipathIndicator + ">";
         }
@@ -846,6 +889,53 @@ public final class GnssMeasurement implements Parcelable {
         mSnrInDb = Double.NaN;
     }
 
+    /**
+     * Returns {@code true} if {@link #getAutomaticGainControlLevelDb()} is available,
+     * {@code false} otherwise.
+     */
+    public boolean hasAutomaticGainControlLevelDb() {
+        return isFlagSet(HAS_AUTOMATIC_GAIN_CONTROL);
+    }
+
+    /**
+     * Gets the Automatic Gain Control level in dB.
+     *
+     * <p> AGC acts as a variable gain amplifier adjusting the power of the incoming signal. The AGC
+     * level may be used to indicate potential interference. When AGC is at a nominal level, this
+     * value must be set as 0. Higher gain (and/or lower input power) shall be output as a positive
+     * number. Hence in cases of strong jamming, in the band of this signal, this value will go more
+     * negative.
+     *
+     * <p>Note: Different hardware designs (e.g. antenna, pre-amplification, or other RF HW
+     * components) may also affect the typical output of of this value on any given hardware design
+     * in an open sky test - the important aspect of this output is that changes in this value are
+     * indicative of changes on input signal power in the frequency band for this measurement.
+     * <p>The value is only available if {@link #hasAutomaticGainControlLevelDb()} is {@code true}
+     */
+    public double getAutomaticGainControlLevelDb() {
+        return mAutomaticGainControlLevelInDb;
+    }
+
+    /**
+     * Sets the Automatic Gain Control level in dB.
+     * @hide
+     */
+    @TestApi
+    public void setAutomaticGainControlLevelInDb(double agcLevelDb) {
+        setFlag(HAS_AUTOMATIC_GAIN_CONTROL);
+        mAutomaticGainControlLevelInDb = agcLevelDb;
+    }
+
+    /**
+     * Resets the Automatic Gain Control level.
+     * @hide
+     */
+    @TestApi
+    public void resetAutomaticGainControlLevel() {
+        resetFlag(HAS_AUTOMATIC_GAIN_CONTROL);
+        mAutomaticGainControlLevelInDb = Double.NaN;
+    }
+
     public static final Creator<GnssMeasurement> CREATOR = new Creator<GnssMeasurement>() {
         @Override
         public GnssMeasurement createFromParcel(Parcel parcel) {
@@ -870,6 +960,7 @@ public final class GnssMeasurement implements Parcelable {
             gnssMeasurement.mCarrierPhaseUncertainty = parcel.readDouble();
             gnssMeasurement.mMultipathIndicator = parcel.readInt();
             gnssMeasurement.mSnrInDb = parcel.readDouble();
+            gnssMeasurement.mAutomaticGainControlLevelInDb = parcel.readDouble();
 
             return gnssMeasurement;
         }
@@ -901,6 +992,7 @@ public final class GnssMeasurement implements Parcelable {
         parcel.writeDouble(mCarrierPhaseUncertainty);
         parcel.writeInt(mMultipathIndicator);
         parcel.writeDouble(mSnrInDb);
+        parcel.writeDouble(mAutomaticGainControlLevelInDb);
     }
 
     @Override
@@ -971,6 +1063,10 @@ public final class GnssMeasurement implements Parcelable {
                 format,
                 "SnrInDb",
                 hasSnrInDb() ? mSnrInDb : null));
+        builder.append(String.format(
+            format,
+            "AgcLevelDb",
+            hasAutomaticGainControlLevelDb() ? mAutomaticGainControlLevelInDb : null));
 
         return builder.toString();
     }
@@ -994,6 +1090,7 @@ public final class GnssMeasurement implements Parcelable {
         resetCarrierPhaseUncertainty();
         setMultipathIndicator(MULTIPATH_INDICATOR_UNKNOWN);
         resetSnrInDb();
+        resetAutomaticGainControlLevel();
     }
 
     private void setFlag(int flag) {
