@@ -17,9 +17,13 @@
 package android.text;
 
 import android.graphics.Paint;
+import android.graphics.Rect;
+import android.text.style.ReplacementSpan;
 import android.text.style.UpdateLayout;
 import android.text.style.WrapTogetherSpan;
+import android.util.ArraySet;
 
+import com.android.internal.annotations.VisibleForTesting;
 import com.android.internal.util.ArrayUtils;
 import com.android.internal.util.GrowingArrayUtils;
 
@@ -81,7 +85,7 @@ public class DynamicLayout extends Layout
         this(base, display, paint, width, align, TextDirectionHeuristics.FIRSTSTRONG_LTR,
                 spacingmult, spacingadd, includepad,
                 StaticLayout.BREAK_STRATEGY_SIMPLE, StaticLayout.HYPHENATION_FREQUENCY_NONE,
-                ellipsize, ellipsizedWidth);
+                Layout.JUSTIFICATION_MODE_NONE, ellipsize, ellipsizedWidth);
     }
 
     /**
@@ -98,7 +102,8 @@ public class DynamicLayout extends Layout
                          int width, Alignment align, TextDirectionHeuristic textDir,
                          float spacingmult, float spacingadd,
                          boolean includepad, int breakStrategy, int hyphenationFrequency,
-                         TextUtils.TruncateAt ellipsize, int ellipsizedWidth) {
+                         int justificationMode, TextUtils.TruncateAt ellipsize,
+                         int ellipsizedWidth) {
         super((ellipsize == null)
                 ? display
                 : (display instanceof Spanned)
@@ -123,6 +128,7 @@ public class DynamicLayout extends Layout
 
         mIncludePad = includepad;
         mBreakStrategy = breakStrategy;
+        mJustificationMode = justificationMode;
         mHyphenationFrequency = hyphenationFrequency;
 
         /*
@@ -296,10 +302,10 @@ public class DynamicLayout extends Layout
                 .setEllipsizedWidth(mEllipsizedWidth)
                 .setEllipsize(mEllipsizeAt)
                 .setBreakStrategy(mBreakStrategy)
-                .setHyphenationFrequency(mHyphenationFrequency);
+                .setHyphenationFrequency(mHyphenationFrequency)
+                .setJustificationMode(mJustificationMode);
         reflowed.generate(b, false, true);
         int n = reflowed.getLineCount();
-
         // If the new layout has a blank line at the end, but it is not
         // the very end of the buffer, then we already have a line that
         // starts there, so disregard the blank line.
@@ -344,9 +350,10 @@ public class DynamicLayout extends Layout
         Directions[] objects = new Directions[1];
 
         for (int i = 0; i < n; i++) {
-            ints[START] = reflowed.getLineStart(i) |
-                          (reflowed.getParagraphDirection(i) << DIR_SHIFT) |
-                          (reflowed.getLineContainsTab(i) ? TAB_MASK : 0);
+            final int start = reflowed.getLineStart(i);
+            ints[START] = start;
+            ints[DIR] |= reflowed.getParagraphDirection(i) << DIR_SHIFT;
+            ints[TAB] |= reflowed.getLineContainsTab(i) ? TAB_MASK : 0;
 
             int top = reflowed.getLineTop(i) + startv;
             if (i > 0)
@@ -360,7 +367,11 @@ public class DynamicLayout extends Layout
             ints[DESCENT] = desc;
             objects[0] = reflowed.getLineDirections(i);
 
-            ints[HYPHEN] = reflowed.getHyphen(i);
+            final int end = (i == n - 1) ? where + after : reflowed.getLineStart(i + 1);
+            ints[HYPHEN] = reflowed.getHyphen(i) & HYPHEN_MASK;
+            ints[MAY_PROTRUDE_FROM_TOP_OR_BOTTOM] |=
+                    contentMayProtrudeFromLineTopOrBottom(text, start, end) ?
+                            MAY_PROTRUDE_FROM_TOP_OR_BOTTOM_MASK : 0;
 
             if (mEllipsize) {
                 ints[ELLIPSIS_START] = reflowed.getEllipsisStart(i);
@@ -380,6 +391,21 @@ public class DynamicLayout extends Layout
         }
     }
 
+    private boolean contentMayProtrudeFromLineTopOrBottom(CharSequence text, int start, int end) {
+        if (text instanceof Spanned) {
+            final Spanned spanned = (Spanned) text;
+            if (spanned.getSpans(start, end, ReplacementSpan.class).length > 0) {
+                return true;
+            }
+        }
+        // Spans other than ReplacementSpan can be ignored because line top and bottom are
+        // disjunction of all tops and bottoms, although it's not optimal.
+        final Paint paint = getPaint();
+        paint.getTextBounds(text, start, end, mTempRect);
+        final Paint.FontMetricsInt fm = paint.getFontMetricsInt();
+        return mTempRect.top < fm.top || mTempRect.bottom > fm.bottom;
+    }
+
     /**
      * Create the initial block structure, cutting the text into blocks of at least
      * BLOCK_MINIMUM_CHARACTER_SIZE characters, aligned on the ends of paragraphs.
@@ -407,6 +433,30 @@ public class DynamicLayout extends Layout
         }
     }
 
+    /**
+     * @hide
+     */
+    public ArraySet<Integer> getBlocksAlwaysNeedToBeRedrawn() {
+        return mBlocksAlwaysNeedToBeRedrawn;
+    }
+
+    private void updateAlwaysNeedsToBeRedrawn(int blockIndex) {
+        int startLine = blockIndex == 0 ? 0 : (mBlockEndLines[blockIndex - 1] + 1);
+        int endLine = mBlockEndLines[blockIndex];
+        for (int i = startLine; i <= endLine; i++) {
+            if (getContentMayProtrudeFromTopOrBottom(i)) {
+                if (mBlocksAlwaysNeedToBeRedrawn == null) {
+                    mBlocksAlwaysNeedToBeRedrawn = new ArraySet<>();
+                }
+                mBlocksAlwaysNeedToBeRedrawn.add(blockIndex);
+                return;
+            }
+        }
+        if (mBlocksAlwaysNeedToBeRedrawn != null) {
+            mBlocksAlwaysNeedToBeRedrawn.remove(blockIndex);
+        }
+    }
+
     /**
      * Create a new block, ending at the specified character offset.
      * A block will actually be created only if has at least one line, i.e. this offset is
@@ -414,11 +464,11 @@ public class DynamicLayout extends Layout
      */
     private void addBlockAtOffset(int offset) {
         final int line = getLineForOffset(offset);
-
         if (mBlockEndLines == null) {
             // Initial creation of the array, no test on previous block ending line
             mBlockEndLines = ArrayUtils.newUnpaddedIntArray(1);
             mBlockEndLines[mNumberOfBlocks] = line;
+            updateAlwaysNeedsToBeRedrawn(mNumberOfBlocks);
             mNumberOfBlocks++;
             return;
         }
@@ -426,6 +476,7 @@ public class DynamicLayout extends Layout
         final int previousBlockEndLine = mBlockEndLines[mNumberOfBlocks - 1];
         if (line > previousBlockEndLine) {
             mBlockEndLines = GrowingArrayUtils.append(mBlockEndLines, mNumberOfBlocks, line);
+            updateAlwaysNeedsToBeRedrawn(mNumberOfBlocks);
             mNumberOfBlocks++;
         }
     }
@@ -440,8 +491,6 @@ public class DynamicLayout extends Layout
      * An index is associated to each block (which will be used by display lists),
      * this class simply invalidates the index of blocks overlapping a modification.
      *
-     * This method is package private and not private so that it can be tested.
-     *
      * @param startLine the first line of the range of modified lines
      * @param endLine the last line of the range, possibly equal to startLine, lower
      * than getLineCount()
@@ -449,7 +498,8 @@ public class DynamicLayout extends Layout
      *
      * @hide
      */
-    void updateBlocks(int startLine, int endLine, int newLineCount) {
+    @VisibleForTesting(visibility = VisibleForTesting.Visibility.PACKAGE)
+    public void updateBlocks(int startLine, int endLine, int newLineCount) {
         if (mBlockEndLines == null) {
             createBlocks();
             return;
@@ -504,13 +554,25 @@ public class DynamicLayout extends Layout
                     blockIndices, firstBlock + numAddedBlocks, mNumberOfBlocks - lastBlock - 1);
             mBlockEndLines = blockEndLines;
             mBlockIndices = blockIndices;
-        } else {
+        } else if (numAddedBlocks + numRemovedBlocks != 0) {
             System.arraycopy(mBlockEndLines, lastBlock + 1,
                     mBlockEndLines, firstBlock + numAddedBlocks, mNumberOfBlocks - lastBlock - 1);
             System.arraycopy(mBlockIndices, lastBlock + 1,
                     mBlockIndices, firstBlock + numAddedBlocks, mNumberOfBlocks - lastBlock - 1);
         }
 
+        if (numAddedBlocks + numRemovedBlocks != 0 && mBlocksAlwaysNeedToBeRedrawn != null) {
+            final ArraySet<Integer> set = new ArraySet<>();
+            for (int i = 0; i < mBlocksAlwaysNeedToBeRedrawn.size(); i++) {
+                Integer block = mBlocksAlwaysNeedToBeRedrawn.valueAt(i);
+                if (block > firstBlock) {
+                    block += numAddedBlocks - numRemovedBlocks;
+                }
+                set.add(block);
+            }
+            mBlocksAlwaysNeedToBeRedrawn = set;
+        }
+
         mNumberOfBlocks = newNumberOfBlocks;
         int newFirstChangedBlock;
         final int deltaLines = newLineCount - (endLine - startLine + 1);
@@ -529,32 +591,40 @@ public class DynamicLayout extends Layout
         int blockIndex = firstBlock;
         if (createBlockBefore) {
             mBlockEndLines[blockIndex] = startLine - 1;
+            updateAlwaysNeedsToBeRedrawn(blockIndex);
             mBlockIndices[blockIndex] = INVALID_BLOCK_INDEX;
             blockIndex++;
         }
 
         if (createBlock) {
             mBlockEndLines[blockIndex] = startLine + newLineCount - 1;
+            updateAlwaysNeedsToBeRedrawn(blockIndex);
             mBlockIndices[blockIndex] = INVALID_BLOCK_INDEX;
             blockIndex++;
         }
 
         if (createBlockAfter) {
             mBlockEndLines[blockIndex] = lastBlockEndLine + deltaLines;
+            updateAlwaysNeedsToBeRedrawn(blockIndex);
             mBlockIndices[blockIndex] = INVALID_BLOCK_INDEX;
         }
     }
 
     /**
-     * This package private method is used for test purposes only
+     * This method is used for test purposes only.
      * @hide
      */
-    void setBlocksDataForTest(int[] blockEndLines, int[] blockIndices, int numberOfBlocks) {
+    @VisibleForTesting(visibility = VisibleForTesting.Visibility.PACKAGE)
+    public void setBlocksDataForTest(int[] blockEndLines, int[] blockIndices, int numberOfBlocks,
+            int totalLines) {
         mBlockEndLines = new int[blockEndLines.length];
         mBlockIndices = new int[blockIndices.length];
         System.arraycopy(blockEndLines, 0, mBlockEndLines, 0, blockEndLines.length);
         System.arraycopy(blockIndices, 0, mBlockIndices, 0, blockIndices.length);
         mNumberOfBlocks = numberOfBlocks;
+        while (mInts.size() < totalLines) {
+            mInts.insertAt(mInts.size(), new int[COLUMNS_NORMAL]);
+        }
     }
 
     /**
@@ -571,6 +641,21 @@ public class DynamicLayout extends Layout
         return mBlockIndices;
     }
 
+    /**
+     * @hide
+     */
+    public int getBlockIndex(int index) {
+        return mBlockIndices[index];
+    }
+
+    /**
+     * @hide
+     * @param index
+     */
+    public void setBlockIndex(int index, int blockIndex) {
+        mBlockIndices[index] = blockIndex;
+    }
+
     /**
      * @hide
      */
@@ -642,7 +727,12 @@ public class DynamicLayout extends Layout
      */
     @Override
     public int getHyphen(int line) {
-        return mInts.getValue(line, HYPHEN);
+        return mInts.getValue(line, HYPHEN) & HYPHEN_MASK;
+    }
+
+    private boolean getContentMayProtrudeFromTopOrBottom(int line) {
+        return (mInts.getValue(line, MAY_PROTRUDE_FROM_TOP_OR_BOTTOM)
+                & MAY_PROTRUDE_FROM_TOP_OR_BOTTOM_MASK) != 0;
     }
 
     @Override
@@ -723,6 +813,7 @@ public class DynamicLayout extends Layout
     private TextUtils.TruncateAt mEllipsizeAt;
     private int mBreakStrategy;
     private int mHyphenationFrequency;
+    private int mJustificationMode;
 
     private PackedIntVector mInts;
     private PackedObjectVector<Directions> mObjects;
@@ -738,6 +829,8 @@ public class DynamicLayout extends Layout
     // The indices of this block's display list in TextView's internal display list array or
     // INVALID_BLOCK_INDEX if this block has been invalidated during an edition
     private int[] mBlockIndices;
+    // Set of blocks that always need to be redrawn.
+    private ArraySet<Integer> mBlocksAlwaysNeedToBeRedrawn;
     // Number of items actually currently being used in the above 2 arrays
     private int mNumberOfBlocks;
     // The first index of the blocks whose locations are changed
@@ -745,17 +838,22 @@ public class DynamicLayout extends Layout
 
     private int mTopPadding, mBottomPadding;
 
+    private Rect mTempRect = new Rect();
+
     private static StaticLayout sStaticLayout = null;
     private static StaticLayout.Builder sBuilder = null;
 
     private static final Object[] sLock = new Object[0];
 
+    // START, DIR, and TAB share the same entry.
     private static final int START = 0;
     private static final int DIR = START;
     private static final int TAB = START;
     private static final int TOP = 1;
     private static final int DESCENT = 2;
+    // HYPHEN and MAY_PROTRUDE_FROM_TOP_OR_BOTTOM share the same entry.
     private static final int HYPHEN = 3;
+    private static final int MAY_PROTRUDE_FROM_TOP_OR_BOTTOM = HYPHEN;
     private static final int COLUMNS_NORMAL = 4;
 
     private static final int ELLIPSIS_START = 4;
@@ -765,6 +863,8 @@ public class DynamicLayout extends Layout
     private static final int START_MASK = 0x1FFFFFFF;
     private static final int DIR_SHIFT  = 30;
     private static final int TAB_MASK   = 0x20000000;
+    private static final int HYPHEN_MASK = 0xFF;
+    private static final int MAY_PROTRUDE_FROM_TOP_OR_BOTTOM_MASK = 0x100;
 
     private static final int ELLIPSIS_UNDEFINED = 0x80000000;
 }
