@@ -24,6 +24,11 @@ import android.graphics.Paint;
 import android.graphics.Rect;
 import android.graphics.drawable.AnimatedVectorDrawable;
 
+import dalvik.annotation.optimization.CriticalNative;
+import dalvik.annotation.optimization.FastNative;
+
+import libcore.util.NativeAllocationRegistry;
+
 /**
  * <p>A display list records a series of graphics related operations and can replay
  * them later. Display lists are usually built by recording operations on a
@@ -51,7 +56,7 @@ import android.graphics.drawable.AnimatedVectorDrawable;
  *
  * <h3>Creating a display list</h3>
  * <pre class="prettyprint">
- *     HardwareRenderer renderer = myView.getHardwareRenderer();
+ *     ThreadedRenderer renderer = myView.getThreadedRenderer();
  *     if (renderer != null) {
  *         DisplayList displayList = renderer.createDisplayList();
  *         DisplayListCanvas canvas = displayList.start(width, height);
@@ -128,17 +133,20 @@ import android.graphics.drawable.AnimatedVectorDrawable;
  */
 public class RenderNode {
 
-    private boolean mValid;
+    // Use a Holder to allow static initialization in the boot image.
+    private static class NoImagePreloadHolder {
+        public static final NativeAllocationRegistry sRegistry = new NativeAllocationRegistry(
+                RenderNode.class.getClassLoader(), nGetNativeFinalizer(), 1024);
+    }
+
     // Do not access directly unless you are ThreadedRenderer
     final long mNativeRenderNode;
     private final View mOwningView;
 
     private RenderNode(String name, View owningView) {
         mNativeRenderNode = nCreate(name);
+        NoImagePreloadHolder.sRegistry.registerNativeAllocation(this, mNativeRenderNode);
         mOwningView = owningView;
-        if (mOwningView instanceof SurfaceView) {
-            nRequestPositionUpdates(mNativeRenderNode, (SurfaceView) mOwningView);
-        }
     }
 
     /**
@@ -146,9 +154,19 @@ public class RenderNode {
      */
     private RenderNode(long nativePtr) {
         mNativeRenderNode = nativePtr;
+        NoImagePreloadHolder.sRegistry.registerNativeAllocation(this, mNativeRenderNode);
         mOwningView = null;
     }
 
+    /**
+     * Immediately destroys the RenderNode
+     * Only suitable for testing/benchmarking where waiting for the GC/finalizer
+     * is not feasible.
+     */
+    public void destroy() {
+        // TODO: Removed temporarily
+    }
+
     /**
      * Creates a new RenderNode that can be used to record batches of
      * drawing operations, and store / apply render properties when drawn.
@@ -171,6 +189,13 @@ public class RenderNode {
         return new RenderNode(nativePtr);
     }
 
+    /**
+     * Enable callbacks for position changes.
+     */
+    public void requestPositionUpdates(SurfaceView view) {
+        nRequestPositionUpdates(mNativeRenderNode, view);
+    }
+
 
     /**
      * Starts recording a display list for the render node. All
@@ -205,7 +230,6 @@ public class RenderNode {
         long displayList = canvas.finishRecording();
         nSetDisplayList(mNativeRenderNode, displayList);
         canvas.recycle();
-        mValid = true;
     }
 
     /**
@@ -214,10 +238,7 @@ public class RenderNode {
      * obsolete resources after related resources are gone.
      */
     public void discardDisplayList() {
-        if (!mValid) return;
-
         nSetDisplayList(mNativeRenderNode, 0);
-        mValid = false;
     }
 
     /**
@@ -226,10 +247,12 @@ public class RenderNode {
      *
      * @return boolean true if the display list is able to be replayed, false otherwise.
      */
-    public boolean isValid() { return mValid; }
+    public boolean isValid() {
+        return nIsValid(mNativeRenderNode);
+    }
 
     long getNativeDisplayList() {
-        if (!mValid) {
+        if (!isValid()) {
             throw new IllegalStateException("The display list is not valid.");
         }
         return mNativeRenderNode;
@@ -767,16 +790,6 @@ public class RenderNode {
         return nGetDebugSize(mNativeRenderNode);
     }
 
-    /**
-     * Called by native when the passed displaylist is removed from the draw tree
-     */
-    void onRenderNodeDetached() {
-        discardDisplayList();
-        if (mOwningView != null) {
-            mOwningView.onRenderNodeDetached(this);
-        }
-    }
-
     ///////////////////////////////////////////////////////////////////////////
     // Animations
     ///////////////////////////////////////////////////////////////////////////
@@ -793,12 +806,12 @@ public class RenderNode {
         return mOwningView != null && mOwningView.mAttachInfo != null;
     }
 
-    public void addAnimator(AnimatedVectorDrawable.VectorDrawableAnimatorRT animatorSet) {
+    public void registerVectorDrawableAnimator(
+            AnimatedVectorDrawable.VectorDrawableAnimatorRT animatorSet) {
         if (mOwningView == null || mOwningView.mAttachInfo == null) {
             throw new IllegalStateException("Cannot start this animator on a detached view!");
         }
-        nAddAnimator(mNativeRenderNode, animatorSet.getAnimatorNativePtr());
-        mOwningView.mAttachInfo.mViewRootImpl.registerAnimatingRenderNode(this);
+        mOwningView.mAttachInfo.mViewRootImpl.registerVectorDrawableAnimator(animatorSet);
     }
 
     public void endAllAnimators() {
@@ -806,105 +819,159 @@ public class RenderNode {
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    // Native methods
+    // Regular JNI methods
     ///////////////////////////////////////////////////////////////////////////
 
-    // Intentionally not static because it acquires a reference to 'this'
-    private native long nCreate(String name);
+    private static native long nCreate(String name);
 
-    private static native void nDestroyRenderNode(long renderNode);
-    private static native void nSetDisplayList(long renderNode, long newData);
-
-    // Matrix
-
-    private static native void nGetTransformMatrix(long renderNode, long nativeMatrix);
-    private static native void nGetInverseTransformMatrix(long renderNode, long nativeMatrix);
-    private static native boolean nHasIdentityMatrix(long renderNode);
-
-    // Properties
-
-    private static native boolean nOffsetTopAndBottom(long renderNode, int offset);
-    private static native boolean nOffsetLeftAndRight(long renderNode, int offset);
-    private static native boolean nSetLeftTopRightBottom(long renderNode, int left, int top,
-            int right, int bottom);
-    private static native boolean nSetBottom(long renderNode, int bottom);
-    private static native boolean nSetRight(long renderNode, int right);
-    private static native boolean nSetTop(long renderNode, int top);
-    private static native boolean nSetLeft(long renderNode, int left);
-    private static native boolean nSetCameraDistance(long renderNode, float distance);
-    private static native boolean nSetPivotY(long renderNode, float pivotY);
-    private static native boolean nSetPivotX(long renderNode, float pivotX);
-    private static native boolean nSetLayerType(long renderNode, int layerType);
-    private static native boolean nSetLayerPaint(long renderNode, long paint);
-    private static native boolean nSetClipToBounds(long renderNode, boolean clipToBounds);
-    private static native boolean nSetClipBounds(long renderNode, int left, int top,
-            int right, int bottom);
-    private static native boolean nSetClipBoundsEmpty(long renderNode);
-    private static native boolean nSetProjectBackwards(long renderNode, boolean shouldProject);
-    private static native boolean nSetProjectionReceiver(long renderNode, boolean shouldRecieve);
-    private static native boolean nSetOutlineRoundRect(long renderNode, int left, int top,
-            int right, int bottom, float radius, float alpha);
-    private static native boolean nSetOutlineConvexPath(long renderNode, long nativePath,
-            float alpha);
-    private static native boolean nSetOutlineEmpty(long renderNode);
-    private static native boolean nSetOutlineNone(long renderNode);
-    private static native boolean nHasShadow(long renderNode);
-    private static native boolean nSetClipToOutline(long renderNode, boolean clipToOutline);
-    private static native boolean nSetRevealClip(long renderNode,
-            boolean shouldClip, float x, float y, float radius);
-    private static native boolean nSetAlpha(long renderNode, float alpha);
-    private static native boolean nSetHasOverlappingRendering(long renderNode,
-            boolean hasOverlappingRendering);
-    private static native boolean nSetElevation(long renderNode, float lift);
-    private static native boolean nSetTranslationX(long renderNode, float translationX);
-    private static native boolean nSetTranslationY(long renderNode, float translationY);
-    private static native boolean nSetTranslationZ(long renderNode, float translationZ);
-    private static native boolean nSetRotation(long renderNode, float rotation);
-    private static native boolean nSetRotationX(long renderNode, float rotationX);
-    private static native boolean nSetRotationY(long renderNode, float rotationY);
-    private static native boolean nSetScaleX(long renderNode, float scaleX);
-    private static native boolean nSetScaleY(long renderNode, float scaleY);
-    private static native boolean nSetStaticMatrix(long renderNode, long nativeMatrix);
-    private static native boolean nSetAnimationMatrix(long renderNode, long animationMatrix);
-
-    private static native boolean nHasOverlappingRendering(long renderNode);
-    private static native boolean nGetClipToOutline(long renderNode);
-    private static native float nGetAlpha(long renderNode);
-    private static native float nGetCameraDistance(long renderNode);
-    private static native float nGetScaleX(long renderNode);
-    private static native float nGetScaleY(long renderNode);
-    private static native float nGetElevation(long renderNode);
-    private static native float nGetTranslationX(long renderNode);
-    private static native float nGetTranslationY(long renderNode);
-    private static native float nGetTranslationZ(long renderNode);
-    private static native float nGetRotation(long renderNode);
-    private static native float nGetRotationX(long renderNode);
-    private static native float nGetRotationY(long renderNode);
-    private static native boolean nIsPivotExplicitlySet(long renderNode);
-    private static native float nGetPivotX(long renderNode);
-    private static native float nGetPivotY(long renderNode);
+    private static native long nGetNativeFinalizer();
     private static native void nOutput(long renderNode);
     private static native int nGetDebugSize(long renderNode);
-
     private static native void nRequestPositionUpdates(long renderNode, SurfaceView callback);
 
-    ///////////////////////////////////////////////////////////////////////////
     // Animations
-    ///////////////////////////////////////////////////////////////////////////
 
     private static native void nAddAnimator(long renderNode, long animatorPtr);
     private static native void nEndAllAnimators(long renderNode);
 
+
     ///////////////////////////////////////////////////////////////////////////
-    // Finalization
+    // @FastNative methods
     ///////////////////////////////////////////////////////////////////////////
 
-    @Override
-    protected void finalize() throws Throwable {
-        try {
-            nDestroyRenderNode(mNativeRenderNode);
-        } finally {
-            super.finalize();
-        }
-    }
+    @FastNative
+    private static native void nSetDisplayList(long renderNode, long newData);
+
+
+    ///////////////////////////////////////////////////////////////////////////
+    // @CriticalNative methods
+    ///////////////////////////////////////////////////////////////////////////
+
+    @CriticalNative
+    private static native boolean nIsValid(long renderNode);
+
+    // Matrix
+
+    @CriticalNative
+    private static native void nGetTransformMatrix(long renderNode, long nativeMatrix);
+    @CriticalNative
+    private static native void nGetInverseTransformMatrix(long renderNode, long nativeMatrix);
+    @CriticalNative
+    private static native boolean nHasIdentityMatrix(long renderNode);
+
+    // Properties
+
+    @CriticalNative
+    private static native boolean nOffsetTopAndBottom(long renderNode, int offset);
+    @CriticalNative
+    private static native boolean nOffsetLeftAndRight(long renderNode, int offset);
+    @CriticalNative
+    private static native boolean nSetLeftTopRightBottom(long renderNode, int left, int top,
+            int right, int bottom);
+    @CriticalNative
+    private static native boolean nSetBottom(long renderNode, int bottom);
+    @CriticalNative
+    private static native boolean nSetRight(long renderNode, int right);
+    @CriticalNative
+    private static native boolean nSetTop(long renderNode, int top);
+    @CriticalNative
+    private static native boolean nSetLeft(long renderNode, int left);
+    @CriticalNative
+    private static native boolean nSetCameraDistance(long renderNode, float distance);
+    @CriticalNative
+    private static native boolean nSetPivotY(long renderNode, float pivotY);
+    @CriticalNative
+    private static native boolean nSetPivotX(long renderNode, float pivotX);
+    @CriticalNative
+    private static native boolean nSetLayerType(long renderNode, int layerType);
+    @CriticalNative
+    private static native boolean nSetLayerPaint(long renderNode, long paint);
+    @CriticalNative
+    private static native boolean nSetClipToBounds(long renderNode, boolean clipToBounds);
+    @CriticalNative
+    private static native boolean nSetClipBounds(long renderNode, int left, int top,
+            int right, int bottom);
+    @CriticalNative
+    private static native boolean nSetClipBoundsEmpty(long renderNode);
+    @CriticalNative
+    private static native boolean nSetProjectBackwards(long renderNode, boolean shouldProject);
+    @CriticalNative
+    private static native boolean nSetProjectionReceiver(long renderNode, boolean shouldRecieve);
+    @CriticalNative
+    private static native boolean nSetOutlineRoundRect(long renderNode, int left, int top,
+            int right, int bottom, float radius, float alpha);
+    @CriticalNative
+    private static native boolean nSetOutlineConvexPath(long renderNode, long nativePath,
+            float alpha);
+    @CriticalNative
+    private static native boolean nSetOutlineEmpty(long renderNode);
+    @CriticalNative
+    private static native boolean nSetOutlineNone(long renderNode);
+    @CriticalNative
+    private static native boolean nHasShadow(long renderNode);
+    @CriticalNative
+    private static native boolean nSetClipToOutline(long renderNode, boolean clipToOutline);
+    @CriticalNative
+    private static native boolean nSetRevealClip(long renderNode,
+            boolean shouldClip, float x, float y, float radius);
+    @CriticalNative
+    private static native boolean nSetAlpha(long renderNode, float alpha);
+    @CriticalNative
+    private static native boolean nSetHasOverlappingRendering(long renderNode,
+            boolean hasOverlappingRendering);
+    @CriticalNative
+    private static native boolean nSetElevation(long renderNode, float lift);
+    @CriticalNative
+    private static native boolean nSetTranslationX(long renderNode, float translationX);
+    @CriticalNative
+    private static native boolean nSetTranslationY(long renderNode, float translationY);
+    @CriticalNative
+    private static native boolean nSetTranslationZ(long renderNode, float translationZ);
+    @CriticalNative
+    private static native boolean nSetRotation(long renderNode, float rotation);
+    @CriticalNative
+    private static native boolean nSetRotationX(long renderNode, float rotationX);
+    @CriticalNative
+    private static native boolean nSetRotationY(long renderNode, float rotationY);
+    @CriticalNative
+    private static native boolean nSetScaleX(long renderNode, float scaleX);
+    @CriticalNative
+    private static native boolean nSetScaleY(long renderNode, float scaleY);
+    @CriticalNative
+    private static native boolean nSetStaticMatrix(long renderNode, long nativeMatrix);
+    @CriticalNative
+    private static native boolean nSetAnimationMatrix(long renderNode, long animationMatrix);
+
+    @CriticalNative
+    private static native boolean nHasOverlappingRendering(long renderNode);
+    @CriticalNative
+    private static native boolean nGetClipToOutline(long renderNode);
+    @CriticalNative
+    private static native float nGetAlpha(long renderNode);
+    @CriticalNative
+    private static native float nGetCameraDistance(long renderNode);
+    @CriticalNative
+    private static native float nGetScaleX(long renderNode);
+    @CriticalNative
+    private static native float nGetScaleY(long renderNode);
+    @CriticalNative
+    private static native float nGetElevation(long renderNode);
+    @CriticalNative
+    private static native float nGetTranslationX(long renderNode);
+    @CriticalNative
+    private static native float nGetTranslationY(long renderNode);
+    @CriticalNative
+    private static native float nGetTranslationZ(long renderNode);
+    @CriticalNative
+    private static native float nGetRotation(long renderNode);
+    @CriticalNative
+    private static native float nGetRotationX(long renderNode);
+    @CriticalNative
+    private static native float nGetRotationY(long renderNode);
+    @CriticalNative
+    private static native boolean nIsPivotExplicitlySet(long renderNode);
+    @CriticalNative
+    private static native float nGetPivotX(long renderNode);
+    @CriticalNative
+    private static native float nGetPivotY(long renderNode);
 }
