@@ -228,6 +228,7 @@ public class TextureView extends View {
      */
     @Override
     protected void destroyHardwareResources() {
+        super.destroyHardwareResources();
         destroyHardwareLayer();
     }
 
@@ -370,11 +371,11 @@ public class TextureView extends View {
 
     HardwareLayer getHardwareLayer() {
         if (mLayer == null) {
-            if (mAttachInfo == null || mAttachInfo.mHardwareRenderer == null) {
+            if (mAttachInfo == null || mAttachInfo.mThreadedRenderer == null) {
                 return null;
             }
 
-            mLayer = mAttachInfo.mHardwareRenderer.createTextureLayer();
+            mLayer = mAttachInfo.mThreadedRenderer.createTextureLayer();
             boolean createNewSurface = (mSurface == null);
             if (createNewSurface) {
                 // Create a new SurfaceTexture for the layer.
@@ -719,7 +720,7 @@ public class TextureView extends View {
     /**
      * Set the {@link SurfaceTexture} for this view to use. If a {@link
      * SurfaceTexture} is already being used by this view, it is immediately
-     * released and not be usable any more.  The {@link
+     * released and not usable any more.  The {@link
      * SurfaceTextureListener#onSurfaceTextureDestroyed} callback is <b>not</b>
      * called for the previous {@link SurfaceTexture}.  Similarly, the {@link
      * SurfaceTextureListener#onSurfaceTextureAvailable} callback is <b>not</b>
