@@ -18,18 +18,18 @@ package android.view;
 
 import android.annotation.IntDef;
 import android.annotation.NonNull;
+import android.app.ActivityManager;
 import android.content.Context;
 import android.content.res.TypedArray;
 import android.graphics.Bitmap;
 import android.graphics.Point;
 import android.graphics.Rect;
-import android.os.Binder;
-import android.os.Handler;
+import android.graphics.drawable.AnimatedVectorDrawable;
 import android.os.IBinder;
-import android.os.Message;
 import android.os.ParcelFileDescriptor;
 import android.os.RemoteException;
 import android.os.ServiceManager;
+import android.os.SystemProperties;
 import android.os.Trace;
 import android.util.Log;
 import android.view.Surface.OutOfResourcesException;
@@ -43,10 +43,9 @@ import java.io.FileDescriptor;
 import java.io.PrintWriter;
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
-import java.util.HashSet;
 
 /**
- * Hardware renderer that proxies the rendering to a render thread. Most calls
+ * Threaded renderer that proxies the rendering to a render thread. Most calls
  * are currently synchronous.
  *
  * The UI thread can block on the RenderThread, but RenderThread must never
@@ -73,18 +72,7 @@ public final class ThreadedRenderer {
     private static final String CACHE_PATH_SHADERS = "com.android.opengl.shaders_cache";
 
     /**
-     * System property used to enable or disable dirty regions invalidation.
-     * This property is only queried if {@link #RENDER_DIRTY_REGIONS} is true.
-     * The default value of this property is assumed to be true.
-     *
-     * Possible values:
-     * "true", to enable partial invalidates
-     * "false", to disable partial invalidates
-     */
-    static final String RENDER_DIRTY_REGIONS_PROPERTY = "debug.hwui.render_dirty_regions";
-
-    /**
-     * System property used to enable or disable hardware rendering profiling.
+     * System property used to enable or disable threaded rendering profiling.
      * The default value of this property is assumed to be false.
      *
      * When profiling is enabled, the adb shell dumpsys gfxinfo command will
@@ -112,7 +100,7 @@ public final class ThreadedRenderer {
 
     /**
      * System property used to specify the number of frames to be used
-     * when doing hardware rendering profiling.
+     * when doing threaded rendering profiling.
      * The default value of this property is #PROFILE_MAX_FRAMES.
      *
      * When profiling is enabled, the adb shell dumpsys gfxinfo command will
@@ -176,6 +164,18 @@ public final class ThreadedRenderer {
      */
     public static final String OVERDRAW_PROPERTY_SHOW = "show";
 
+    /**
+     * Defines the rendering pipeline to be used by the ThreadedRenderer.
+     *
+     * Possible values:
+     * "opengl", will use the existing OpenGL renderer
+     * "skiagl", will use Skia's OpenGL renderer
+     * "skiavk", will use Skia's Vulkan renderer
+     *
+     * @hide
+     */
+    public static final String DEBUG_RENDERER_PROPERTY = "debug.hwui.renderer";
+
     /**
      * Turn on to debug non-rectangular clip operations.
      *
@@ -189,8 +189,13 @@ public final class ThreadedRenderer {
     public static final String DEBUG_SHOW_NON_RECTANGULAR_CLIP_PROPERTY =
             "debug.hwui.show_non_rect_clip";
 
+    static {
+        // Try to check OpenGL support early if possible.
+        isAvailable();
+    }
+
     /**
-     * A process can set this flag to false to prevent the use of hardware
+     * A process can set this flag to false to prevent the use of threaded
      * rendering.
      *
      * @hide
@@ -198,14 +203,14 @@ public final class ThreadedRenderer {
     public static boolean sRendererDisabled = false;
 
     /**
-     * Further hardware renderer disabling for the system process.
+     * Further threaded renderer disabling for the system process.
      *
      * @hide
      */
     public static boolean sSystemRendererDisabled = false;
 
     /**
-     * Invoke this method to disable hardware rendering in the current process.
+     * Invoke this method to disable threaded rendering in the current process.
      *
      * @hide
      */
@@ -219,28 +224,47 @@ public final class ThreadedRenderer {
     public static boolean sTrimForeground = false;
 
     /**
-     * Controls whether or not the hardware renderer should aggressively
-     * trim memory. Note that this must not be set for any process that
-     * uses WebView! This should be only used by system_process or similar
+     * Controls whether or not the renderer should aggressively trim
+     * memory. Note that this must not be set for any process that uses
+     * WebView! This should be only used by system_process or similar
      * that do not go into the background.
      */
     public static void enableForegroundTrimming() {
         sTrimForeground = true;
     }
 
+    private static Boolean sSupportsOpenGL;
+
     /**
-     * Indicates whether hardware acceleration is available under any form for
+     * Indicates whether threaded rendering is available under any form for
      * the view hierarchy.
      *
-     * @return True if the view hierarchy can potentially be hardware accelerated,
+     * @return True if the view hierarchy can potentially be defer rendered,
      *         false otherwise
      */
     public static boolean isAvailable() {
-        return DisplayListCanvas.isAvailable();
+        if (sSupportsOpenGL != null) {
+            return sSupportsOpenGL.booleanValue();
+        }
+        if (SystemProperties.getInt("ro.kernel.qemu", 0) == 0) {
+            // Device is not an emulator.
+            sSupportsOpenGL = true;
+            return true;
+        }
+        int qemu_gles = SystemProperties.getInt("qemu.gles", -1);
+        if (qemu_gles == -1) {
+            // In this case, the value of the qemu.gles property is not ready
+            // because the SurfaceFlinger service may not start at this point.
+            return false;
+        }
+        // In the emulator this property will be set > 0 when OpenGL ES 2.0 is
+        // enabled, 0 otherwise. On old emulator versions it will be undefined.
+        sSupportsOpenGL = qemu_gles > 0;
+        return sSupportsOpenGL.booleanValue();
     }
 
     /**
-     * Sets the directory to use as a persistent storage for hardware rendering
+     * Sets the directory to use as a persistent storage for threaded rendering
      * resources.
      *
      * @param cacheDir A directory the current process can write to
@@ -252,16 +276,16 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Creates a hardware renderer using OpenGL.
+     * Creates a threaded renderer using OpenGL.
      *
      * @param translucent True if the surface is translucent, false otherwise
      *
-     * @return A hardware renderer backed by OpenGL.
+     * @return A threaded renderer backed by OpenGL.
      */
-    public static ThreadedRenderer create(Context context, boolean translucent) {
+    public static ThreadedRenderer create(Context context, boolean translucent, String name) {
         ThreadedRenderer renderer = null;
-        if (DisplayListCanvas.isAvailable()) {
-            renderer = new ThreadedRenderer(context, translucent);
+        if (isAvailable()) {
+            renderer = new ThreadedRenderer(context, translucent, name);
         }
         return renderer;
     }
@@ -285,10 +309,6 @@ public final class ThreadedRenderer {
         nOverrideProperty(name, value);
     }
 
-    public static void dumpProfileData(byte[] data, FileDescriptor fd) {
-        nDumpProfileData(data, fd);
-    }
-
     // Keep in sync with DrawFrameTask.h SYNC_* flags
     // Nothing interesting to report
     private static final int SYNC_OK = 0;
@@ -338,13 +358,13 @@ public final class ThreadedRenderer {
     private long mNativeProxy;
     private boolean mInitialized = false;
     private RenderNode mRootNode;
-    private Choreographer mChoreographer;
     private boolean mRootNodeNeedsUpdate;
 
     private boolean mEnabled;
     private boolean mRequested = true;
+    private boolean mIsOpaque = false;
 
-    ThreadedRenderer(Context context, boolean translucent) {
+    ThreadedRenderer(Context context, boolean translucent, String name) {
         final TypedArray a = context.obtainStyledAttributes(null, R.styleable.Lighting, 0, 0);
         mLightY = a.getDimension(R.styleable.Lighting_lightY, 0);
         mLightZ = a.getDimension(R.styleable.Lighting_lightZ, 0);
@@ -357,7 +377,9 @@ public final class ThreadedRenderer {
         long rootNodePtr = nCreateRootRenderNode();
         mRootNode = RenderNode.adopt(rootNodePtr);
         mRootNode.setClipToBounds(false);
+        mIsOpaque = !translucent;
         mNativeProxy = nCreateProxy(translucent, rootNodePtr);
+        nSetName(mNativeProxy, name);
 
         ProcessInitializer.sInstance.init(context, mNativeProxy);
 
@@ -365,7 +387,7 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Destroys the hardware rendering context.
+     * Destroys the threaded rendering context.
      */
     void destroy() {
         mInitialized = false;
@@ -374,25 +396,25 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Indicates whether hardware acceleration is currently enabled.
+     * Indicates whether threaded rendering is currently enabled.
      *
-     * @return True if hardware acceleration is in use, false otherwise.
+     * @return True if threaded rendering  is in use, false otherwise.
      */
     boolean isEnabled() {
         return mEnabled;
     }
 
     /**
-     * Indicates whether hardware acceleration is currently enabled.
+     * Indicates whether threaded rendering  is currently enabled.
      *
-     * @param enabled True if the hardware renderer is in use, false otherwise.
+     * @param enabled True if the threaded renderer is in use, false otherwise.
      */
     void setEnabled(boolean enabled) {
         mEnabled = enabled;
     }
 
     /**
-     * Indicates whether hardware acceleration is currently request but not
+     * Indicates whether threaded rendering is currently request but not
      * necessarily enabled yet.
      *
      * @return True if requested, false otherwise.
@@ -402,10 +424,8 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Indicates whether hardware acceleration is currently requested but not
+     * Indicates whether threaded rendering is currently requested but not
      * necessarily enabled yet.
-     *
-     * @return True to request hardware acceleration, false otherwise.
      */
     void setRequested(boolean requested) {
         mRequested = requested;
@@ -420,9 +440,9 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Initializes the hardware renderer for the specified surface.
+     * Initializes the threaded renderer for the specified surface.
      *
-     * @param surface The surface to hardware accelerate
+     * @param surface The surface to render
      *
      * @return True if the initialization was successful, false otherwise.
      */
@@ -435,16 +455,16 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Initializes the hardware renderer for the specified surface and setup the
+     * Initializes the threaded renderer for the specified surface and setup the
      * renderer for drawing, if needed. This is invoked when the ViewAncestor has
-     * potentially lost the hardware renderer. The hardware renderer should be
+     * potentially lost the threaded renderer. The threaded renderer should be
      * reinitialized and setup when the render {@link #isRequested()} and
      * {@link #isEnabled()}.
      *
      * @param width The width of the drawing surface.
      * @param height The height of the drawing surface.
      * @param attachInfo Information about the window.
-     * @param surface The surface to hardware accelerate
+     * @param surface The surface to render
      * @param surfaceInsets The drawing surface insets to apply
      *
      * @return true if the surface was initialized, false otherwise. Returning
@@ -465,9 +485,9 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Updates the hardware renderer for the specified surface.
+     * Updates the threaded renderer for the specified surface.
      *
-     * @param surface The surface to hardware accelerate
+     * @param surface The surface to render
      */
     void updateSurface(Surface surface) throws OutOfResourcesException {
         updateEnabledState(surface);
@@ -476,7 +496,7 @@ public final class ThreadedRenderer {
 
     /**
      * Halts any current rendering into the surface. Use this if it is unclear whether
-     * or not the surface used by the HardwareRenderer will be changing. It
+     * or not the surface used by the ThreadedRenderer will be changing. It
      * Suspends any rendering into the surface, but will not do any destruction.
      *
      * Any subsequent draws will override the pause, resuming normal operation.
@@ -506,25 +526,6 @@ public final class ThreadedRenderer {
 
     private static void destroyResources(View view) {
         view.destroyHardwareResources();
-
-        if (view instanceof ViewGroup) {
-            ViewGroup group = (ViewGroup) view;
-
-            int count = group.getChildCount();
-            for (int i = 0; i < count; i++) {
-                destroyResources(group.getChildAt(i));
-            }
-        }
-    }
-
-    /**
-     * This method should be invoked whenever the current hardware renderer
-     * context should be reset.
-     *
-     * @param surface The surface to hardware accelerate
-     */
-    void invalidate(Surface surface) {
-        updateSurface(surface);
     }
 
     /**
@@ -566,7 +567,7 @@ public final class ThreadedRenderer {
         }
 
         mRootNode.setLeftTopRightBottom(-mInsetLeft, -mInsetTop, mSurfaceWidth, mSurfaceHeight);
-        nSetup(mNativeProxy, mSurfaceWidth, mSurfaceHeight, mLightRadius,
+        nSetup(mNativeProxy, mLightRadius,
                 mAmbientShadowAlpha, mSpotShadowAlpha);
 
         setLightCenter(attachInfo);
@@ -588,10 +589,22 @@ public final class ThreadedRenderer {
     }
 
     /**
-     * Change the HardwareRenderer's opacity
+     * Change the ThreadedRenderer's opacity
      */
     void setOpaque(boolean opaque) {
-        nSetOpaque(mNativeProxy, opaque && !mHasInsets);
+        mIsOpaque = opaque && !mHasInsets;
+        nSetOpaque(mNativeProxy, mIsOpaque);
+    }
+
+    boolean isOpaque() {
+        return mIsOpaque;
+    }
+
+    /**
+     * Enable/disable wide gamut rendering on this renderer.
+     */
+    void setWideGamut(boolean wideGamut) {
+        nSetWideGamut(mNativeProxy, wideGamut);
     }
 
     /**
@@ -657,7 +670,7 @@ public final class ThreadedRenderer {
         view.mRecreateDisplayList = false;
     }
 
-    private void updateRootDisplayList(View view, HardwareDrawCallbacks callbacks) {
+    private void updateRootDisplayList(View view, DrawCallbacks callbacks) {
         Trace.traceBegin(Trace.TRACE_TAG_VIEW, "Record View#draw()");
         updateViewTreeDisplayList(view);
 
@@ -666,13 +679,13 @@ public final class ThreadedRenderer {
             try {
                 final int saveCount = canvas.save();
                 canvas.translate(mInsetLeft, mInsetTop);
-                callbacks.onHardwarePreDraw(canvas);
+                callbacks.onPreDraw(canvas);
 
                 canvas.insertReorderBarrier();
                 canvas.drawRenderNode(view.updateDisplayListIfDirty());
                 canvas.insertInorderBarrier();
 
-                callbacks.onHardwarePostDraw(canvas);
+                callbacks.onPostDraw(canvas);
                 canvas.restoreToCount(saveCount);
                 mRootNodeNeedsUpdate = false;
             } finally {
@@ -725,29 +738,29 @@ public final class ThreadedRenderer {
 
     /**
      * Interface used to receive callbacks whenever a view is drawn by
-     * a hardware renderer instance.
+     * a threaded renderer instance.
      */
-    interface HardwareDrawCallbacks {
+    interface DrawCallbacks {
         /**
-         * Invoked before a view is drawn by a hardware renderer.
+         * Invoked before a view is drawn by a threaded renderer.
          * This method can be used to apply transformations to the
          * canvas but no drawing command should be issued.
          *
          * @param canvas The Canvas used to render the view.
          */
-        void onHardwarePreDraw(DisplayListCanvas canvas);
+        void onPreDraw(DisplayListCanvas canvas);
 
         /**
-         * Invoked after a view is drawn by a hardware renderer.
+         * Invoked after a view is drawn by a threaded renderer.
          * It is safe to invoke drawing commands from this method.
          *
          * @param canvas The Canvas used to render the view.
          */
-        void onHardwarePostDraw(DisplayListCanvas canvas);
+        void onPostDraw(DisplayListCanvas canvas);
     }
 
     /**
-     *  Indicates that the content drawn by HardwareDrawCallbacks needs to
+     *  Indicates that the content drawn by DrawCallbacks needs to
      *  be updated, which will be done by the next call to draw()
      */
     void invalidateRoot() {
@@ -761,7 +774,7 @@ public final class ThreadedRenderer {
      * @param attachInfo AttachInfo tied to the specified view.
      * @param callbacks Callbacks invoked when drawing happens.
      */
-    void draw(View view, AttachInfo attachInfo, HardwareDrawCallbacks callbacks) {
+    void draw(View view, AttachInfo attachInfo, DrawCallbacks callbacks) {
         attachInfo.mIgnoreDirtyState = true;
 
         final Choreographer choreographer = attachInfo.mViewRootImpl.mChoreographer;
@@ -843,15 +856,6 @@ public final class ThreadedRenderer {
         nCancelLayerUpdate(mNativeProxy, layer.getDeferredLayerUpdater());
     }
 
-    /**
-     * Optional, sets the name of the renderer. Useful for debugging purposes.
-     *
-     * @param name The name of this renderer, can be null
-     */
-    void setName(String name) {
-        nSetName(mNativeProxy, name);
-    }
-
     /**
      * Blocks until all previously queued work has completed.
      */
@@ -881,12 +885,33 @@ public final class ThreadedRenderer {
         nRegisterAnimatingRenderNode(mRootNode.mNativeRenderNode, animator.mNativeRenderNode);
     }
 
+    void registerVectorDrawableAnimator(
+        AnimatedVectorDrawable.VectorDrawableAnimatorRT animator) {
+        nRegisterVectorDrawableAnimator(mRootNode.mNativeRenderNode,
+                animator.getAnimatorNativePtr());
+    }
+
     public void serializeDisplayListTree() {
         nSerializeDisplayListTree(mNativeProxy);
     }
 
-    public static int copySurfaceInto(Surface surface, Bitmap bitmap) {
-        return nCopySurfaceInto(surface, bitmap);
+    public static int copySurfaceInto(Surface surface, Rect srcRect, Bitmap bitmap) {
+        if (srcRect == null) {
+            // Empty rect means entire surface
+            return nCopySurfaceInto(surface, 0, 0, 0, 0, bitmap);
+        } else {
+            return nCopySurfaceInto(surface, srcRect.left, srcRect.top,
+                    srcRect.right, srcRect.bottom, bitmap);
+        }
+    }
+
+    /**
+     * Creates a {@link android.graphics.Bitmap.Config#HARDWARE} bitmap from the given
+     * RenderNode. Note that the RenderNode should be created as a root node (so x/y of 0,0), and
+     * not the RenderNode from a View.
+     **/
+    public static Bitmap createHardwareBitmap(RenderNode node, int width, int height) {
+        return nCreateHardwareBitmap(node.getNativeDisplayList(), width, height);
     }
 
     @Override
@@ -901,59 +926,62 @@ public final class ThreadedRenderer {
 
     private static class ProcessInitializer {
         static ProcessInitializer sInstance = new ProcessInitializer();
-        private static IBinder sProcToken;
 
         private boolean mInitialized = false;
 
+        private Context mAppContext;
+        private IGraphicsStats mGraphicsStatsService;
+        private IGraphicsStatsCallback mGraphicsStatsCallback = new IGraphicsStatsCallback.Stub() {
+            @Override
+            public void onRotateGraphicsStatsBuffer() throws RemoteException {
+                rotateBuffer();
+            }
+        };
+
         private ProcessInitializer() {}
 
         synchronized void init(Context context, long renderProxy) {
             if (mInitialized) return;
             mInitialized = true;
-            initGraphicsStats(context, renderProxy);
-            initAssetAtlas(context, renderProxy);
+            mAppContext = context.getApplicationContext();
+            initSched(renderProxy);
+            initGraphicsStats();
         }
 
-        private static void initGraphicsStats(Context context, long renderProxy) {
+        private void initSched(long renderProxy) {
+            try {
+                int tid = nGetRenderThreadTid(renderProxy);
+                ActivityManager.getService().setRenderThread(tid);
+            } catch (Throwable t) {
+                Log.w(LOG_TAG, "Failed to set scheduler for RenderThread", t);
+            }
+        }
+
+        private void initGraphicsStats() {
             try {
                 IBinder binder = ServiceManager.getService("graphicsstats");
                 if (binder == null) return;
-                IGraphicsStats graphicsStatsService = IGraphicsStats.Stub
-                        .asInterface(binder);
-                sProcToken = new Binder();
-                final String pkg = context.getApplicationInfo().packageName;
-                ParcelFileDescriptor pfd = graphicsStatsService.
-                        requestBufferForProcess(pkg, sProcToken);
-                nSetProcessStatsBuffer(renderProxy, pfd.getFd());
-                pfd.close();
+                mGraphicsStatsService = IGraphicsStats.Stub.asInterface(binder);
+                requestBuffer();
             } catch (Throwable t) {
                 Log.w(LOG_TAG, "Could not acquire gfx stats buffer", t);
             }
         }
 
-        private static void initAssetAtlas(Context context, long renderProxy) {
-            IBinder binder = ServiceManager.getService("assetatlas");
-            if (binder == null) return;
+        private void rotateBuffer() {
+            nRotateProcessStatsBuffer();
+            requestBuffer();
+        }
 
-            IAssetAtlas atlas = IAssetAtlas.Stub.asInterface(binder);
+        private void requestBuffer() {
             try {
-                if (atlas.isCompatible(android.os.Process.myPpid())) {
-                    GraphicBuffer buffer = atlas.getBuffer();
-                    if (buffer != null) {
-                        long[] map = atlas.getMap();
-                        if (map != null) {
-                            nSetAtlas(renderProxy, buffer, map);
-                        }
-                        // If IAssetAtlas is not the same class as the IBinder
-                        // we are using a remote service and we can safely
-                        // destroy the graphic buffer
-                        if (atlas.getClass() != binder.getClass()) {
-                            buffer.destroy();
-                        }
-                    }
-                }
-            } catch (RemoteException e) {
-                Log.w(LOG_TAG, "Could not acquire atlas", e);
+                final String pkg = mAppContext.getApplicationInfo().packageName;
+                ParcelFileDescriptor pfd = mGraphicsStatsService
+                        .requestBufferForProcess(pkg, mGraphicsStatsCallback);
+                nSetProcessStatsBuffer(pfd.getFd());
+                pfd.close();
+            } catch (Throwable t) {
+                Log.w(LOG_TAG, "Could not acquire gfx stats buffer", t);
             }
         }
     }
@@ -968,10 +996,14 @@ public final class ThreadedRenderer {
         observer.mNative = null;
     }
 
+    /** Not actually public - internal use only. This doc to make lint happy */
+    public static native void disableVsync();
+
     static native void setupShadersDiskCache(String cacheFile);
 
-    private static native void nSetAtlas(long nativeProxy, GraphicBuffer buffer, long[] map);
-    private static native void nSetProcessStatsBuffer(long nativeProxy, int fd);
+    private static native void nRotateProcessStatsBuffer();
+    private static native void nSetProcessStatsBuffer(int fd);
+    private static native int nGetRenderThreadTid(long nativeProxy);
 
     private static native long nCreateRootRenderNode();
     private static native long nCreateProxy(boolean translucent, long rootRenderNode);
@@ -984,14 +1016,16 @@ public final class ThreadedRenderer {
     private static native void nUpdateSurface(long nativeProxy, Surface window);
     private static native boolean nPauseSurface(long nativeProxy, Surface window);
     private static native void nSetStopped(long nativeProxy, boolean stopped);
-    private static native void nSetup(long nativeProxy, int width, int height,
+    private static native void nSetup(long nativeProxy,
             float lightRadius, int ambientShadowAlpha, int spotShadowAlpha);
     private static native void nSetLightCenter(long nativeProxy,
             float lightX, float lightY, float lightZ);
     private static native void nSetOpaque(long nativeProxy, boolean opaque);
+    private static native void nSetWideGamut(long nativeProxy, boolean wideGamut);
     private static native int nSyncAndDrawFrame(long nativeProxy, long[] frameInfo, int size);
     private static native void nDestroy(long nativeProxy, long rootRenderNode);
     private static native void nRegisterAnimatingRenderNode(long rootRenderNode, long animatingNode);
+    private static native void nRegisterVectorDrawableAnimator(long rootRenderNode, long animator);
 
     private static native void nInvokeFunctor(long functor, boolean waitForCompletion);
 
@@ -1014,7 +1048,6 @@ public final class ThreadedRenderer {
 
     private static native void nDumpProfileInfo(long nativeProxy, FileDescriptor fd,
             @DumpFlags int dumpFlags);
-    private static native void nDumpProfileData(byte[] data, FileDescriptor fd);
 
     private static native void nAddRenderNode(long nativeProxy, long rootRenderNode,
              boolean placeFront);
@@ -1026,5 +1059,8 @@ public final class ThreadedRenderer {
     private static native long nAddFrameMetricsObserver(long nativeProxy, FrameMetricsObserver observer);
     private static native void nRemoveFrameMetricsObserver(long nativeProxy, long nativeObserver);
 
-    private static native int nCopySurfaceInto(Surface surface, Bitmap bitmap);
+    private static native int nCopySurfaceInto(Surface surface,
+            int srcLeft, int srcTop, int srcRight, int srcBottom, Bitmap bitmap);
+
+    private static native Bitmap nCreateHardwareBitmap(long renderNode, int width, int height);
 }
