@@ -38,6 +38,7 @@ import android.util.Property;
 import android.util.TypedValue;
 import android.view.Gravity;
 import android.view.MotionEvent;
+import android.view.PointerIcon;
 import android.view.View;
 import android.view.View.MeasureSpec;
 import android.view.ViewConfiguration;
@@ -1441,6 +1442,13 @@ class FastScroller {
         return false;
     }
 
+    public PointerIcon onResolvePointerIcon(MotionEvent event, int pointerIndex) {
+        if (mState == STATE_DRAGGING || isPointInside(event.getX(), event.getY())) {
+            return PointerIcon.getSystemIcon(mList.getContext(), PointerIcon.TYPE_ARROW);
+        }
+        return null;
+    }
+
     public boolean onTouchEvent(MotionEvent me) {
         if (!isEnabled()) {
             return false;
