@@ -277,18 +277,19 @@ public class SpellChecker implements SpellCheckerSessionListener {
             // Do not check this word if the user is currently editing it
             final boolean isEditing;
 
-            // Defer spell check when typing a word with an interior apostrophe.
-            // TODO: a better solution to this would be to make the word
-            // iterator locale-sensitive and include the apostrophe in
-            // languages that use it (such as English).
-            final boolean apostrophe = (selectionStart == end + 1 && editable.charAt(end) == '\'');
-            if (mIsSentenceSpellCheckSupported) {
+            // Defer spell check when typing a word ending with a punctuation like an apostrophe
+            // which could end up being a mid-word punctuation.
+            if (selectionStart == end + 1
+                    && WordIterator.isMidWordPunctuation(
+                            mCurrentLocale, Character.codePointBefore(editable, end + 1))) {
+                isEditing = false;
+            } else if (mIsSentenceSpellCheckSupported) {
                 // Allow the overlap of the cursor and the first boundary of the spell check span
                 // no to skip the spell check of the following word because the
                 // following word will never be spell-checked even if the user finishes composing
-                isEditing = !apostrophe && (selectionEnd <= start || selectionStart > end);
+                isEditing = selectionEnd <= start || selectionStart > end;
             } else {
-                isEditing = !apostrophe && (selectionEnd < start || selectionStart > end);
+                isEditing = selectionEnd < start || selectionStart > end;
             }
             if (start >= 0 && end > start && isEditing) {
                 spellCheckSpan.setSpellCheckInProgress(true);
