@@ -33,7 +33,9 @@
 #include "HarfBuzzNGFaceSkia.h"
 
 #include <stdlib.h>
-#include <cutils/log.h>
+
+#include <log/log.h>
+
 #include <SkPaint.h>
 #include <SkPath.h>
 #include <SkPoint.h>
@@ -50,7 +52,7 @@ static const bool kDebugGlyphs = false;
 // calls. See the Harfbuzz source for references about what these callbacks do.
 
 struct HarfBuzzFontData {
-    HarfBuzzFontData(SkPaint* paint) : m_paint(paint) { }
+    explicit HarfBuzzFontData(SkPaint* paint) : m_paint(paint) { }
     SkPaint* m_paint;
 };
 
